% Proc INIT.m
% Set up grid for capital and productivity,
% calculate transition probs,
% Initialize parameters common to all experiments

% SIZE OF GRID
ggz = 11;
m   = 3;          		 % Spread in Z grid (+- std. devs.)


% PARAMETER VALUES
zeta        = 25;

sigma     	= 0.21;      % SD of productivity shocks 
rho 	    = 0.90;	     % Persistence of productivity shocks
sigma_perm	= 0.34;      % SD of permanent wage differences

alfa 	    = 0.3;       % Target for labor share of total time
theta   	= 0.3;	     % Capital share in production
g 		    = 0.0;		 % Growth rate in economy
delta 	    = 0.075;     % Depreciation rate of physical capital
beta        = 0.95;      % Discount factor
mu		    = 1.5;       % Risk aversion
G           = 0;

gmma        = 2;         % inverse of Frish labor supply elasticity
zeta        = 30;        % parameter in utility fcn

% SET UP STATE SPACE

kmax = ((alfa^(1-theta)) / delta)^(1/(1-theta));
kmax = 6*kmax;

% Assets
minA = 0;
maxA = kmax;
A = [0 .001 .003 .005 .01 .015 .02 .03 .04 .05 .1:.1:1 1.2:.2:2 2.5:.5:10 11:1:17 20 25:5:50 70 90];

% Productivity endowment
[Z, Zprob] = Tauchen(ggz,0,rho,sigma,m);
Z          = Z';
distr      = (ones(1,ggz)/ggz) * Zprob^1000;
Ql         = exp(Z - sigma_perm);
Qh         = exp(Z + sigma_perm);
EQ         = distr * (Ql' + Qh') / 2;
Ql         = Ql / EQ; Qh = Qh / EQ;
Q          = [Ql Qh];
bigZprob   = [Zprob zeros(size(Zprob)); zeros(size(Zprob)) Zprob];

params     = [alfa beta gmma mu zeta g theta delta G];

gga = length(A);
ggz = length(Z);
