% Estimate frisch elasticity from artificial data
% Use "realistic" sample size if FULL_SAMPLE = 0

clear all

SEPUTIL              = 1;
USE_ALL_IN_1ST_STAGE = 0;
ADDITIONAL_INFO      = 0;
FULL_SAMPLE          = 1;
nESTIMATES           = 500;

if FULL_SAMPLE
    nESTIMATES = 2;
end

tt=1:400;

if SEPUTIL                                % Separable utility
load simsav
nZ   = length(Ql);
wbar = w;
Asim = [Asiml(:,tt); Asimh(:,tt)];
Hsim = [Hsiml(:,tt); Hsimh(:,tt)];
Csim = [Csiml(:,tt); Csimh(:,tt)];
Qsim = [Qsiml(:,tt); Qsimh(:,tt)];
iZsim = [iZsiml(:,tt); iZsimh(:,tt)+nZ];
else                                       % Cobb Douglas utility
%load simsav_web_cd
load simsav_cd
wbar = w;
Asim = [Asiml(:,tt); Asimh(:,tt)];
Hsim = [Hsiml(:,tt); Hsimh(:,tt)];
Csim = [Csiml(:,tt); Csimh(:,tt)];
Qsim = [Qsiml(:,tt); Qsimh(:,tt)];
Hsim = 1-Hsim; 
iZsim = zeros(size(Qsim)); 
QQ = [Q_low Q_hgh];
for i = 1:length(Qsim(:)), iZsim(i) = find(Qsim(i)==QQ); end
end

Csim0 = Csim; Qsim0 = Qsim;

STHb  = 0;
SIV1b = 0;
SIV2b = 0;
SLEVb = 0;
SSLb  = 0;

for II = 1:nESTIMATES

% Choose 3 years and select all individuals in those years
[nAg,T] = size(Asim);
ss  = [];
for ii = 0:2
  year = floor(rand*100) +  50 + ii*125;
  ss = [ss; ((1:nAg)' + year*nAg)];
end

if FULL_SAMPLE
ss = ((2*nAg+1):(nAg*T))';             % include full sample?
end

Xsim = log(Qsim(ss-nAg))-log(Qsim0(ss-nAg));
varX = std(Xsim(:))^2;


h      = log(Hsim(ss));
c      = log(Csim(ss));
w      = log(Qsim(ss));
lag_h  = log(Hsim(ss-nAg));
lag_w  = log(Qsim(ss-nAg));
lag2_w = log(Qsim(ss-2*nAg));
c      = log(Csim(ss));
lag_c  = log(Csim(ss-nAg));
iZ_lag = iZsim(ss-nAg);

DH     = Hsim(ss)-Hsim(ss-nAg);
Dh     = h - lag_h;
Dc     = c - lag_c;
Dw     = w - lag_w;
lag_Dw = lag_w - lag2_w;

Abar   = mean(mean(Asim(ss)));
A      = Asim(ss);
Ax     = (A<Abar/5);
Ay     = (A<Abar/20);


ii     = find(A>(-.1)*Abar);
n      = length(Dw(ii));
regress
sTHr2  = [THr2 n]; sIV1r2 = [IV1r2 n]; sIV2r2 = [IV2r2 n]; sLEVr2 = [LEVr2 n]; sSLr2=[SLr2 n];
sTHb   = [THb THst];
sIV1b  = [IV1b IV1st];
sIV2b  = [IV2b IV2st];
sLEVb  = [LEVb LEVst];
sSLb   = [SLb SLst];
USE_ALL_IN_1ST_STAGE = 0;
save_pred_dw = pred_dw;


ii     = find(A>.01*Abar);
n      = length(Dw(ii));
regress
sTHr2  = [sTHr2 THr2 n];
sIV1r2 = [sIV1r2 IV1r2 n];
sIV2r2 = [sIV2r2 IV2r2 n];
sLEVr2 = [sLEVr2 LEVr2 n];
sSLr2  = [sSLr2 SLr2 n];
sTHb   = [sTHb THb THst];
sIV1b  = [sIV1b IV1b IV1st];
sIV2b  = [sIV2b IV2b IV2st];
sLEVb  = [sLEVb LEVb LEVst];
sSLb   = [sSLb SLb SLst];

ii     = find(A<.1*Abar);
n      = length(Dw(ii));
regress
sTHr2  = [sTHr2 THr2 n];
sIV1r2 = [sIV1r2 IV1r2 n];
sIV2r2 = [sIV2r2 IV2r2 n];
sLEVr2 = [sLEVr2 LEVr2 n];
sSLr2  = [sSLr2 SLr2 n];
sTHb   = [sTHb THb THst];
sIV1b  = [sIV1b IV1b IV1st];
sIV2b  = [sIV2b IV2b IV2st];
sLEVb  = [sLEVb LEVb LEVst];
sSLb   = [sSLb SLb SLst];

ii     = find(A>.01*Abar & A<1*Abar);
n      = length(Dw(ii));
regress
sTHr2  = [sTHr2 THr2 n];
sIV1r2 = [sIV1r2 IV1r2 n];
sIV2r2 = [sIV2r2 IV2r2 n];
sLEVr2 = [sLEVr2 LEVr2 n];
sSLr2  = [sSLr2 SLr2 n];
sTHb   = [sTHb THb THst];
sIV1b  = [sIV1b IV1b IV1st];
sIV2b  = [sIV2b IV2b IV2st];
sLEVb  = [sLEVb LEVb LEVst];
sSLb   = [sSLb SLb SLst];

ii     = find(A>1*Abar);
n      = length(Dw(ii));
regress
STHr2(II,:)  = [sTHr2 THr2 n];
SIV1r2(II,:) = [sIV1r2 IV1r2 n];
SIV2r2(II,:) = [sIV2r2 IV2r2 n];
SLEVr2(II,:) = [sLEVr2 LEVr2 n];
SSLr2(II,:)  = [sSLr2 SLr2 n];
STHb         = STHb + [sTHb THb THst] / nESTIMATES;
SIV1b        = SIV1b + [sIV1b IV1b IV1st] / nESTIMATES;
SIV2b        = SIV2b + [sIV2b IV2b IV2st] / nESTIMATES;
SLEVb        = SLEVb + [sLEVb LEVb LEVst] / nESTIMATES;
SSLb         = SSLb + [sSLb SLb SLst] / nESTIMATES;

F_TH(II)  = THf;
F_IV1(II) = IV1f;
F_IV2(II) = IV2f;
F_LEV(II) = LEVf;
F_SL(II)  = SLf;

end

fprintf(1,'Theoretical instrument:\n');
fprintf(1,'%-6.4f %8.0f   ',mean(STHr2)); fprintf(1,'\n');
fprintf(1,'%-14.2f    ',mean(F_TH)); fprintf(1,'\n');
fprintf(1,'%8.4f %6.4f : %8.4f %6.4f : %8.4f %6.4f : %8.4f %6.4f : %8.4f %6.4f\n',STHb')

fprintf(1,'\nFirst stage IV:\n');
fprintf(1,'%-6.4f %8.0f   ',mean(SIV1r2)); fprintf(1,'\n'); 
fprintf(1,'%-14.2f    ',mean(F_IV1)); fprintf(1,'\n');
fprintf(1,'%8.4f %6.4f : %8.4f %6.4f : %8.4f %6.4f : %8.4f %6.4f : %8.4f %6.4f\n',SIV1b')

fprintf(1,'\nSecond stage IV:\n');
fprintf(1,'%-6.4f %8.0f   ',mean(SIV2r2)); fprintf(1,'\n');
fprintf(1,'%-14.2f    ',mean(F_IV2)); fprintf(1,'\n');
fprintf(1,'%8.4f %6.4f : %8.4f %6.4f : %8.4f %6.4f : %8.4f %6.4f : %8.4f %6.4f\n',SIV2b')

fprintf(1,'\nLevels:\n');
fprintf(1,'%-6.4f %8.0f   ',mean(SLEVr2)); fprintf(1,'\n');
fprintf(1,'%-14.2f    ',mean(F_LEV)); fprintf(1,'\n');
fprintf(1,'%8.4f %6.4f : %8.4f %6.4f : %8.4f %6.4f : %8.4f %6.4f : %8.4f %6.4f\n',SLEVb')

fprintf(1,'\nSemi Log:\n');
fprintf(1,'%-6.4f %8.0f   ',mean(SSLr2)); fprintf(1,'\n');
fprintf(1,'%-14.2f    ',mean(F_SL)); fprintf(1,'\n');
fprintf(1,'%8.4f %6.4f : %8.4f %6.4f : %8.4f %6.4f : %8.4f %6.4f : %8.4f %6.4f\n',SSLb')



if ADDITIONAL_INFO
  % Calculate some statistics based on the sample
  Earnings = wbar * Qsim(ss) .* Hsim(ss);
  Income   = Earnings + Asim(ss)*r;
  Wealth   = Asim(ss);
  Wage     = wbar * Qsim(ss);
  Hours    = Hsim(ss);

  N        = length(Earnings);
  N40      = floor(0.4*N);
  N10      = floor(0.9*N);
  N1       = floor(0.99*N);

  Wsum     = sum(Wealth);
  Wsort    = sort(Wealth);
  Wfrac40  = sum(Wsort(1:N40)) / Wsum
  Wfrac10  = sum(Wsort(N10:end)) / Wsum
  Wfrac1   = sum(Wsort(N1:end)) / Wsum
  Wgini    = gini(Wealth)
end



