% Report statistics. 
b=0; K=Kbar;
  
Cs = [Csiml; Csimh];
Hs = [Hsiml; Hsimh];
Qs = [Qsiml; Qsimh];
As = [Asiml; Asimh];
HQs = Hs.*Qs; 
  
  CC = mean(mean(Cs));
  HQ = mean(mean(HQs));
  HH = mean(mean(Hs));
  YY = K^(theta/(1-theta)) * HQ;   % productivity normalized to unity
  
  
  % Correlations of hours, wages and assets
  hh0 = Hs(:); 
  aa0 = As(:);
  ww0 = w*Qs(:);
  ccHW = corrcoef(hh0,ww0);
  ccHA = corrcoef(hh0,aa0);
  ccWA =  corrcoef(ww0,aa0);
  clear ww0 hh0
  ccHW = ccHW(1,2); ccHA = ccHA(1,2); ccWA = ccWA(1,2);
ee0 = w*Qs(:).*Hs(:);
ii0 = ee0 + b + r * As(:);
ccEA = corrcoef(ee0,aa0);
ccEI = corrcoef(ee0,ii0);
ccIA = corrcoef(ii0,aa0);
ccEA = ccEA(1,2); ccEI = ccEI(1,2); ccIA = ccIA(1,2);
 
clear ee0 ii0 aa0

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate distributional implications and volatilities
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
FIGURES = 0;
LORENZ = 0;
VOLATILS = 1;
nn = length(Cs(:));

% Calculate labor earings before tax = 'Earnings'
Es = [Hs .* Qs * w];

% Calculate net factor income before tax but after transfers = 'Total Income'
Is = Es + r * As;
H0 = [Hs(1:nn)];


% DISTRIBUTIONS 

C0 = sort(Cs(1:nn));
A0 = sort(As(1:nn));
E0 = sort(Es(1:nn));
I0 = sort(Is(1:nn));

gC = gini(C0);
gA = gini(A0);
gE = gini(E0);
gI = gini(I0);


c40 = sum(C0(1:floor(nn*.4))) / sum(C0);
a40 = sum(A0(1:floor(nn*.4))) / sum(A0);
e40 = sum(E0(1:floor(nn*.4))) / sum(E0);
i40 = sum(I0(1:floor(nn*.4))) / sum(I0);
  
c20 = sum(C0(floor(nn*.80):nn)) / sum(C0);
a20 = sum(A0(floor(nn*.80):nn)) / sum(A0);
e20 = sum(E0(floor(nn*.80):nn)) / sum(E0);
i20 = sum(I0(floor(nn*.80):nn)) / sum(I0);
  
c10 = sum(C0(floor(nn*.90):nn)) / sum(C0);
a10 = sum(A0(floor(nn*.90):nn)) / sum(A0);
e10 = sum(E0(floor(nn*.90):nn)) / sum(E0);
i10 = sum(I0(floor(nn*.90):nn)) / sum(I0);

c1  = sum(C0(floor(nn*.99):nn)) / sum(C0);
a1  = sum(A0(floor(nn*.99):nn)) / sum(A0);
e1  = sum(E0(floor(nn*.99):nn)) / sum(E0);
i1  = sum(I0(floor(nn*.99):nn)) / sum(I0);
  
if FIGURES
  hh = figure;
  set(hh,'PaperType','a4letter');
  XX = -0.5:0.1:5;
  cc = hist(C0,XX);
  gy = hist(Y0,XX);
  nn2 = sum(XX <= 4);
  plot(XX(1:nn2),cc(1:nn2)/nn,'-',XX(1:nn2),gy(1:nn2)/nn,':');
  legend('consumption','net disp. income');
  title('Distributions');
  xlabel('c, y');
  ylabel('Density');


  hh = figure;
  set(hh,'PaperType','a4letter');
  AA = 0:0.2:A(gga);
  nn0 = size(AA,2) / 0.2;
  N = hist(A0,AA);
  nn2 = sum(AA <= 10);
  plot(AA(1:nn2),N(1:nn2)/nn0);
  title('Asset distribution');
  xlabel('A');
  ylabel('Density');
end;

if LORENZ
  indx = 100*(1:nn)/nn;
  plot(indx,indx,indx,100*cumsum(A0)/sum(A0),indx,100*cumsum(E0)/sum(E0));
  xlabel('% of individuals');
  ylabel('% of total');
  gtext('Wealth');
  gtext('Earnings');

end;

if VOLATILS
  std_c    = mean(std(Cs(:))/mean(Cs(:)));
  std_a    = mean(std(As(:))/mean(As(:)));
  std_h    = mean(std(Hs(:))/mean(Hs(:)));
  std_e    = mean(std(E0)/mean(E0));
  std_i    = mean(std(I0)/mean(I0));
end;


diary off;
diary('stats.out');
  fprintf('\n===========================================================================');  
  fprintf('%s    ',date); cl = clock;
  fprintf('%2i:%2i\n',cl(4),cl(5));
  fprintf('SET UP:\n',0);
  fprintf('mu      = %10.6f\n',mu);
  fprintf('rho     = %10.6f,rho');
  fprintf('   sigma   = %10.6f',sigma);
  fprintf('   alfa    = %10.6f\n',alfa);
  fprintf('gmma    = %10.6f',gmma);
  fprintf('   beta    = %10.6f\n',beta);
  fprintf('#A      = %10i',gga);
  fprintf('   #Z      = %10i\n',ggz);
  fprintf('\nRESULTS:\n');
  fprintf('r       = %10.6f',r);
  fprintf('   w       = %10.6f\n',w);
  fprintf('K       = %10.6f',K);
  fprintf('   A       = %10.6f\n',Abar);
  fprintf('H       = %10.6f\n',HH);
  fprintf('Hbar    = %10.6f',Hbar);
  fprintf('   HQ      = %10.6f\n',HQ);
  fprintf('C       = %10.6f',CC);
  fprintf('   Y       = %10.6f\n',YY);
  fprintf('sd%%(C)  = %10.6f',std_c);
  fprintf('   sd%%(A)  = %10.6f',std_a);
  fprintf('   sd%%(H)  = %10.6f\n',std_h);
  fprintf('sd%%(E)  = %10.6f',std_e); 
  fprintf('   sd%%(I)  = %10.6f\n',std_i);
  fprintf('cr(h,w) = %10.6f',ccHW);
  fprintf('   cr(h,a) = %10.6f',ccHA);
  fprintf('   cr(w,a) = %10.6f\n',ccWA);
  fprintf('cr(e,a) = %10.6f',ccEA);
  fprintf('   cr(e,i) = %10.6f',ccEI);
  fprintf('   cr(i,a) = %10.6f\n',ccIA);
  fprintf('gini(C) = %10.6f',gC);
  fprintf('   gini(A) = %10.6f\n',gA);
  fprintf('gini(E) = %10.6f',gE);
  fprintf('   gini(I) = %10.6f\n',gI);
  fprintf('\n');
  fprintf('(A40- A20+ A10+ A1+) = %8.4f %8.4f %8.4f %8.4f\n',a40,a20,a10,a1);
  fprintf('(E40- E20+ E10+ E1+) = %8.4f %8.4f %8.4f %8.4f\n',e40,e20,e10,e1);
  fprintf('(I40- I20+ I10+ I1+) = %8.4f %8.4f %8.4f %8.4f\n',i40,i20,i10,i1);
  fprintf('\nTESTS OF SOLUTION: \n');
  fprintf('max(a)  = %10.6f',max(max(As)));
  fprintf('   max(A)  = %10.6f',A(gga));
  fprintf('   #a>.9A  = %10.6f\n',sum(sum(As>.9*A(gga))));
  fprintf('===========================================================================\n');  
