function [Abar, Hbar, maxAs, Asim, Hsim, Csim, iZsim] = ...
   simulate(A_dec,H_dec,Q,Zprob,A,Kbar,B,rr,ww,A1,iZ);
% SIMULATE the economy for one agent
% Asim is wealth in beginning of period

nA = size(A,2);
nZ = size(Q,2);

fprintf(1,'Simulating...\n');

nPeriods = 400;
nAg = 250;						% Simulate nAg agents at a time

bigA = ones(nAg,1) * A;

x = cumsum(Zprob')';
x = [zeros(nZ,1) x];
 
As = zeros(nPeriods,nAg);
maxAs = 0;

% Initiate distributions to get fast convergence
if (nAg ~= length(A1)) | (nAg ~= length(iZ))
  fprintf('initiating for simulations\n')
  A1 = Kbar * ones(nAg,1);
  iZ = floor(nZ/2) * ones(nAg,1);
  iZ(1) = 1;
  iZ(nAg) = nZ;
  nSkip = 1500;
else nSkip = 1500;
end

n = nSkip + nPeriods;

EA  = 0;
HQ = 0;


Asim = zeros(nAg,nPeriods);
Hsim = Asim; Csim = Asim; iZsim = Asim;

% START SIMULATION

for j = 1:n;
  
  A0 = A1;	
  
  % Update Z and Q
  iZ = sum((x(iZ,:) < (rand(nAg,1) * ones(1,nZ+1)))')';
  Q0 = Q(iZ)';
       
  % Choose A(t). Interpolate in A-dimension
  xlow = sum(((A0 * ones(1,nA)) >= bigA)')';
  xlow = max(min(xlow,nA-1),1);
  xhigh = xlow+1;
  ilow = nA * (iZ-1) + xlow;
  ihigh = ilow + 1;
  frac = (A(xhigh)' - A0) ./ (A(xhigh)' - A(xlow)');
  A1 = frac .* A_dec(ilow) + (1-frac) .* A_dec(ihigh);
  H = frac .* H_dec(ilow) + (1-frac) .* H_dec(ihigh);
  HQ0 = H .* Q0;
  
if min(A1) < -1e-6
  fprints(1,'min(A1) = %10.6f\n',min(A1));
end
  A1 = max(A1,-1e-8);
  
  if j > nSkip
    EA  = EA + mean(A0) / nPeriods;
    HQ  = HQ + mean(HQ0) / nPeriods;
    ma = max(A0);
    if ma > maxAs
      maxAs = ma;
    end;
    Asim(:,j-nSkip)=A0;
    Hsim(:,j-nSkip)=H;
    Csim(:,j-nSkip)=B + (1+rr)*A0 + ww*HQ0 - A1;
    iZsim(:,j-nSkip)=iZ;
  end      
end

% END OF SIMULATION

Abar = EA;
Hbar = HQ;