function [A_dec, C_dec, H_dec] = ...
   solve_foc(A,Q,Pr,A_dec,C_dec,H_dec,ww,rr,B,Kbar,params)
% ---------------------------------------------------------------------------
% SOLVE_FOC
% u = c^(1-mu) / (1-mu) - zeta*h^(1+gmma) / (1+gmma)
% 
% 010207
% Solve first order condition
% Iterate on f.o.c.
%
% Pr = transition matrix
% A_dec = guess for solution for a'
% w = after tax wage per efficiency unit
% r = after tax interest rate
% b = lump sum transfer
% ---------------------------------------------------------------------------

beta = params(2);
gmma = params(3);
mu   = params(4);
zeta = params(5);

R = 1 + rr;

nA = length(A);
nZ = length(Q);


maxFOCdiff = 1e-4;
maxFOCiters = 150;
maxEEdiff = 1e-8;
maxEEiters = 50;
repinter = 50;


% ITERATE ON FOC TO SOLVE FOR VALUE FUNCTION

% Some calculations and initiations used in the loop

toc1 = toc;

bigA = A' * ones(1,nZ*nA);
smallNum = 1e-8;

% Vectorize: v = { (z1,a1), (z2,a1), ..., (znZ,a1), (z1,a2), ... }
vecA0 = ones(nZ,1) * A;
vecA0 = vecA0(:)';
vecQ = Q' * ones(1,nA);
vecQ = vecQ(:)';
vecAdec = A_dec; vecAdec = vecAdec(:)';
vecCdec = C_dec; vecCdec = vecCdec(:)';
vecHdec = H_dec; vecHdec = vecHdec(:)';
%nextHdec0 = zeros(nZ,nA);
nextHdec = zeros(nZ,nA*nZ);
nextCdec = nextHdec;
nextAdec = nextHdec;
bigTransit = zeros(nZ,nZ*nA);
for i_ = 1:nA;
  ind_ = 1+(i_-1)*nZ;
  bigTransit(:,ind_:ind_+(nZ-1)) = Pr';
end;

FOCdiff = 1;
FOCiter = 1;

while (FOCiter <= maxFOCiters) & (FOCdiff > maxFOCdiff)
  
  preAdec = vecAdec;
  preHdec = vecHdec;
  nextA1 	= vecAdec;	 	% Initial guess: a' = a
  vecH	= vecHdec;		% Guess h(a',a,q)
  minA    = max(nextA1 - max(Kbar/2,nextA1/3),-smallNum);
  maxA    = R * vecA0 + B + ww * vecQ;
  firstMin = minA; firstMax = maxA;
  diffMin = .5 * ones(1,nZ*nA);
  diffMax = -1 * ones(1,nZ*nA);
  
  % Solve Euler eqn. given next period's decicion rules vecAdec, vecHdec, and vecCdec
  EEdiff = 1;
  EEiter = 1;
  while (EEdiff > maxEEdiff) & (EEiter <= maxEEiters)
    
    nextA = nextA1;

    [vecC, vecH] = solve_CH(vecA0,vecQ,nextA,rr,ww,B,params);


    
    % Now, evaluate the LHS in the EE:
    LHS = vecC.^(-mu) + pi * min(0,nextA);
    
    % Evaluate RHS
    % Build nZ * (nZ*nA) matrix of Hdec(a',q') etc.
    % Must interpolate since a' not on grid
    
      % 1. For each a' in nextA, calculate the largest i such that A(i) < a'
      %    Corner solutions --> i = 1 or i = nA-1
      ind = sum(ones(nA,1)*nextA > bigA);
      ind = min(max(ind,1),nA-1);
    
      % 2. Calculate fraction f such that f = 1 if A(i+1) = a'
      frac = (nextA - A(ind)) ./ (A(ind+1) - A(ind));
%      frac = min(max(frac,0),1);

      % 3. Interpolate decisions for each possible q'
      for FOCiZ = 1:nZ
        nextHdec(FOCiZ,:) = frac .* vecHdec(ind*nZ+FOCiZ) + ...
          (1-frac) .* vecHdec((ind-1)*nZ+FOCiZ);
        nextHdec(FOCiZ,:) = max(nextHdec(FOCiZ,:),0);
        nextCdec(FOCiZ,:) = frac .* vecCdec(ind*nZ+FOCiZ) + ...
          (1-frac) .* vecCdec((ind-1)*nZ+FOCiZ);
        nextCdec(FOCiZ,:) = max(nextCdec(FOCiZ,:),1e-4);
      end;
      bigRHS = nextCdec.^(-mu);
      RHS = beta * R * sum(bigTransit .* bigRHS);
      
    % Compare LHS and RHS and update a' with secant method if iter >=3 else half step
    direct = (RHS > LHS);
    corner = (nextA1 < 1e-4) & (direct == 0);
    diffEE = (RHS - LHS).*(1-corner);  
    EEdiff = mean(abs(diffEE));
    [maxdiff, imd] = max(abs(diffEE));
    minA = direct .* nextA + (1-direct) .* minA;
    maxA = (1-direct) .* nextA + direct .* maxA;
    diffMin = direct .* diffEE + (1-direct) .* diffMin;
    diffMax = (1-direct) .* diffEE + direct .* diffMax;
    
    if EEiter < 4
      nextA1 = direct .* (nextA + maxA) / 2 + (1-direct) .* (nextA + minA) / 2;
    else
      diffdiff = diffMin - diffMax;
      diffdiff = diffdiff + (diffdiff == 0) .* ones(1,nZ*nA);
      diffstep = diffMin ./ diffdiff;
      diffcorn = nextA1 < 1e-4;
      diffstep = diffcorn .* min(max(diffstep,.25),.75) + (1-diffcorn) .* diffstep;
      nextA1 = minA + diffstep .* (maxA-minA);
    end;
    if (EEiter > 5)
      corner0 = (minA == (-smallNum));
      corner1 = (minA == firstMin);
      corner2 = (maxA == firstMax);
      diffMin(corner1) = diffMin(corner1) .* 1e-1;
      diffMax(corner2) = diffMax(corner2) .* 1e-1;
      nextA1 =  corner0 .* min(minA + smallNum,nextA1) + (1-corner0) .* nextA1;
%      nextA1 =  corner2 .* max(maxA - smallNum,nextA1) + (1-corner2) .* nextA1;
    end;
    
    EEiter = EEiter + 1;
  end;
  

  if EEdiff > maxEEdiff, fprintf('EE did not converge: diff = %14.10f,',EEdiff);
    fprintf(' FOCiter = %5i',FOCiter);
    fprintf(' Grid point = %5i\n',imd); end;
  
  if mean(abs(nextA1-nextA)) > 1e-6, warning('Warning: Solution for A unstable in EE'); end;
  vecAdec = nextA;
  vecHdec = vecH;
  vecCdec = vecC;
  
  FOCdiff = (vecAdec-preAdec);
  FOCdiff(nA*nZ-nZ:nA*nZ) = FOCdiff(nA*nZ-nZ:nA*nZ)/20;
  FOCdiff = mean(abs(FOCdiff)) + mean(abs(vecHdec-preHdec));
       
  toc2 = toc;
  if rem(FOCiter,repinter) == 0
     fprintf('Iter: %4i',FOCiter);
     fprintf('   Diff: %10.6f',FOCdiff);
     fprintf('   Elapsed time: %8.2f',toc2);
     fprintf('   Last loop: %5.2f\n',toc2-toc1);
  end;
  toc1 = toc2;

  FOCiter = FOCiter + 1;
  
end; % iters

% Set A_dec = 0 if this holds approximately
vecAdec = vecAdec .* (vecAdec > smallNum);

fprintf('Solved f.o.c. on iteration %4i',FOCiter-1);
fprintf('. Diff = %10.6f\n',FOCdiff);

	
A_dec = reshape(vecAdec,nZ,nA);
C_dec = reshape(vecCdec,nZ,nA);
H_dec = reshape(vecHdec,nZ,nA);



% Function solve_CH
function [C, H] = solve_CH(a,q,anxt,r,w,b,params)

gmma = params(3);
mu   = params(4); 
zeta = params(5);


C = (1+r)*a + .04*w*q + b - anxt;
C = max(C,1e-4);

df = 1;
it = 1;
while (df > 1e-8) & (it < 100)
  oldC = C;
  wqh = w*q.*(w.*q./((C.^mu)*zeta)).^(1/gmma);
  f = C - b - wqh - (1+r) * a + anxt;
  fprime = 1 + (mu/gmma) * wqh ./ C;
  C = C - f ./ fprime;
  C = max(C,1e-4);

  df = sum(abs(C - oldC)) + (min(C)<2e-3);
  it = it + 1;
end

if df > 1e-5, prints('C, H did not converge: diff = '); prints(df,14,10,0); end;

H = ((w*q) ./ (zeta*(C.^mu))).^(1/gmma);
oldH=H;
%H = min(H,1);
 
test1 = b + (1+r) * a + w*q.*H - C;

diff1 = max(abs(anxt-test1));
if diff1 > 1e-6
  fprintf('Diff = %10.6f, iter = %4i, minC = %10.6f, maxC = %10.6f\n',diff1,it-1,min(C),max(C));
end


test2 = w.*q.*C.^(-mu) - zeta*H.^gmma;
diff2 = max(abs(test2));

if diff2 > 1e-6
  fprintf('diff2 = %10.6f, H>1 = %10.6f\n',diff2,max(oldH));
end
 